function [ mf_nh3, Q_generator ] = abs_ref( T_amb, Q_e, Q_el, T_el, ...
                                                Q_hx2, T_hx2, Q_IC, T_IC )
% ABS_REF   Calculating the required mass flow of NH3 to comply the
%           cooling demand.
%           Calculating the heat input required for the generator.
%           The heat from heat exchanger 2 and the intercooler in 
%           compressor 1 is used to heat up the cooling
%           media applied in the internal electrolyzer heat exchanger 
%           to the required input temperature, T_input.
%           The cooling medium is water. The heating medium is steam.
%           The steam is assumed to be saturated at T_input.
%           The temperature in the evaporator is assumed to be -45 degrees.
%   
% INPUT
% T_amb     Ambient temperature, [K]
% Q_e       Heat that needs to be removed by the evaporator, [kW]
% Q_el      Heat removed in eletrolysis, [kW]
% T_el      Operating temperature of electrolyzer, [K]
% Q_hx2     Heat removed in heat exchanger 2, [kW]
% T_hx2     Outlet temperatureof cooling water in HX2, [K]
% Q_IC      Heat removed in intercooling in compressor 1, [kW]
% T_IC      Intermediate temperature in compressor 1, [K]
%
% OUTPUT
% mf_nh3        Required mass flow of refrigerant NH3, [kg/s]
% Q_generator   Required heat input to the generator, [kW]

%%%%%%%%%%%%%%%%%%%% PARAMETERS %%%%%%%%%%%%%%%%%%%%%%
T_cw            = T_amb;            %Cooling water temperature

%Coefficient of performance based on cooling water temepratures
if T_cw < 273 + 20
    COP         = 0.45;
elseif 273 + 20 <= T_cw && T_cw < 273 + 30
    COP         = 0.4;
else
    COP         = 0.3;
end

%Required input temperature and corresponding enthalpies based on 
%cooling temperatures.
if T_cw <= 273 + 15
    %T_input     = 273 + 150;
    p_cw        = 0.48;             %Saturation pressure, [MPa]
    
    %Enthalpies of water at 4.8 bar and 150 degrees C, [kJ/kg]
    %(Saturated state)
    h_l_cw      = 633.5;
    h_v_cw      = 2746;
    
elseif 273 + 15 < T_cw <= 273 + 20
    %T_input     = 273 + 160;
    p_cw        = 0.62;         
    
    %Enthalpies of water at 6.2 bar and 160 degrees, [kJ/kg] 
    h_l_cw      = 670.4;    
    h_v_cw      = 2756;
    
elseif 273 + 21 <= T_cw <= 273 + 25
    %T_input     = 273 + 175;
    p_cw        = 0.9;
    
    %Enthalpies of water at 9 bar and 175 degrees, [kJ/kg]
    h_l_cw      = 742.6;
    h_v_cw      = 2772;
    
else
    %T_input     = 273 + 186;
    p_cw        = 1.19;
    
    %Enthalpies of water at 11.9 bar and 186 degrees, [kJ/kg]
    h_l_cw      = 790;
    h_v_cw      = 2781;
end

%Cooling water 
h_cw_0          = Props('H', 'P', p_cw*1e3, 'T', T_cw, 'H2O'); 

%%%%%%%%%%%%%%%%%%% EVAPORATOR %%%%%%%%%%%%%%%%%%%%%%%%%%%

%Specific enthalpy for liquid and vapor ammonia, [kJ/kg]. Ref: NIST 
h_l_nh3         = -965.06;      % T = -45degC, p = 0.545 bar
h_v_nh3         = 437.67;                        

%Energy balance evaporator where Q_e is the cooling demand.
%The required ammonia mass flow in the refrigeration cycle is calculated.
mf_nh3          = Q_e/(h_v_nh3 - h_l_nh3);

%%%%%%%%%%%%%%%%%% GENERATOR %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Required heat input in generator
Q_generator     = Q_e/COP;

%Required steam mass flow
mf_gen          = Q_generator/(h_v_cw - h_l_cw);

disp(['Required mass flow of steam is ', num2str(mf_gen), ' kg/s'])

%%%%%%%%%%%%%%%%%% ELECTROLYZER %%%%%%%%%%%%%%%%%%%%%%%%%%%
%Need to heat the cooling water coming from the electrolyzer to T_input.
%Therefore, it is necessary to calculate how much heat is needed
%to produce the required steam flow, mf_gen.

%Enthalpies
h_l_el          = Props('H', 'P', p_cw*1e3, ...     %Hot water from elect.
                                            'T', T_el, 'H2O'); 
h_v_el          = h_v_cw;       %After heating up to T_input

%Available mass flow from electrolyzer
mf_el           = Q_el/(h_l_el - h_cw_0);

if mf_el < mf_gen
    disp('Mass flow from electrolyzer cooling is not sufficient')
    return      %Break if mass flow is not sufficient
else 
    disp(['Available mass flow from electrolyzer cooling is ', ...
                                        num2str(mf_el), ' kg/s']);
end

%Required heat from hx2 to heat up to necessary temperature
Q_req_hx2       = mf_gen*(h_v_el - h_l_el);

%%%%%%%%%%%%%%%%%% HEAT EXCHANGER 2 %%%%%%%%%%%%%%%%%%%%%%

%Enthalpies
h_l_hx2         = h_cw_0;                   %Cooling water at ambient temp.
h_v_hx2         = Props('H', 'P', p_cw*1e3, ...
                        'T', T_hx2, 'H2O'); %Outlet cooling water conditions

%Mass flow of available water for heating the electrolyzer flow
mf_hx2          = Q_hx2/(h_v_hx2 - h_l_hx2);

%Heat that can be used for heating at the required temperature
Q_hx2_avail     = -mf_hx2*(h_l_cw - h_v_hx2);

%Mass flow of water required from heat exchanger 2
mf_req_hx2      = -Q_req_hx2/(h_l_el - h_v_hx2);

disp(['The required mass flow from heat exchanger 2 is ', ...
                                num2str(mf_req_hx2), ' kg/s'])
disp(['The available mass flow from heat exchanger 2 is ', ...
                                num2str(mf_hx2), ' kg/s'])
                            
%%%%%%%%%%%%%%%%% HEAT EXCHANGER 2 + INTERCOOLER %%%%%%%%%%%
h_l_ic          = h_cw_0;                        %Inlet conditions intercooler
h_v_ic          = Props('H', 'T', T_IC, ...      %Outlet conditions intercooler
                                        'P', p_cw*1e3, 'H2O'); 

%Mass flow in intercooler
mf_ic           = Q_IC/(h_v_ic - h_l_ic);

%Heat that can be used for heating at the required temperature 
Q_ic_avail      = -mf_ic*(h_l_cw - h_v_ic);

%Mixing these two flows together
mf_hx2_ic       = mf_ic + mf_hx2;
h_hx2_ic        = (mf_ic*h_v_ic + mf_hx2*h_v_hx2)/mf_hx2_ic;
T_hx2_ic        = Props('T', 'P', p_cw*1e3, ...    %Combined flow temperature
                                            'H', h_hx2_ic, 'H2O'); 

%Required mass flow from heat exchanger 2 + intercooler
mf_req_hx2_ic   = -Q_req_hx2/(h_l_el - h_hx2_ic);

disp(['The required mass flow from heat exchanger 2 plus the intercooler is '...
                                        , num2str(mf_req_hx2_ic), ' kg/s'])
disp(['The available mass flow from heat exchanger 2 plus the intercooler is ' ...
                                        , num2str(mf_hx2_ic), ' kg/s'])                                    



end
