function [ W, mflow, T_out, T_int1, Q_int ] = ...
                            compressor1( p_in, p_out, T_in, T_out_last ) 
% COMPRESSOR1   Calculates the shaft work for compressor 1. The
%               compression is considered adiabatic, and the work 
%               is calculated from the energy balance:        
%                       W   =   m*( h_out(T,p) - h_in(T,p) )
%               The outlet enthalpy, h_out is calculated from the 
%               isentropic efficiency and the outlet isentropic enthalpy:
%                       h_out   = ( h_out,is - h_in )/eta_is + h_in
%               The isentropic enthalpy is then calculated from the 
%               outlet entropy and pressure.
%               The compression is performed in two stages with
%               intercooling.
%               The compression ratio in the first stage is 5.
%
% INPUT
% p_in          Inlet pressure, [MPa]
% p_out         Outlet pressure, [MPa]
% T_in          Inlet temperature, [K]
% T_out_last    Outlet temperature, [K]
% 
% OUTPUT
% W             Compressor work, [kW]
% mflow         Mass flow, [kg/s]
% T_out         Outlet temperature, [K]
% T_int1        Intermediate temperature, [K]
% Q_int         Heat removed in intercooling, [Q]

global T_amb

%Parameters
Vcyl        = 2.3e-3;           %Cylinder volume for both compressors, [m3] 
c           = 10;               %Cylinder clearance, [%]
mflow       = 0.017;            %Constant mass flow, [kg/s]

%Heat capacities
Cp          = Props('C', 'P', p_in*1e3, 'T', T_in, 'H2');
Cv          = Props('O', 'P', p_in*1e3, 'T', T_in, 'H2');
gamma       = Cp/Cv;

%Compressibility factors
Zin         = Props('Z', 'T', T_in, 'P', p_in*1e3, 'H2');
Zout        = Props('Z', 'T', T_out_last, 'P', p_out*1e3, 'H2');

%%%%%%%%%% EFFICIENCIES AND TRANSPORT %%%%%%%%%%%%%%%%%

eta_v       = @(p1, p2)     (100 - p2/p1 - c*((p2/p1)^(1/gamma)...
                            *Zin^2/Zout^2 - 1))*1e-2;
eta_is      = @(p1, p2)     0.1091*(log(p2/p1))^3 - ...
                            0.5247*(log(p2/p1))^2 + 0.8577*log(p2/p1)+ ...
                            0.3727;                        
speed       = @(eta, rho)   mflow/(Vcyl*rho*eta);

%%%%%%%%% TWO STAGE COMPRESSION %%%%%%%%%%%%%%%%%%%%%
r           = 5;                %Compression ratio
p_int       = r*p_in;           %Intermediate pressure

%%%%%%%%% INLET CONDITIONS %%%%%%%%%%%%%%%%%

% Calculate inlet enthalpy and entropy from known T and p.
h_in        = Props('H', 'T', T_in, 'P', p_in*1e3, 'H2');
s_in        = Props('S', 'T', T_in, 'P', p_in*1e3, 'H2');
rho_in      = Props('D', 'T', T_in, 'P', p_in*1e3, 'H2'); 

%%%%%%%% MASS FLOW %%%%%%%%%%%%%%%%%%%%%%%%

N           = speed(eta_v(p_in, p_int), rho_in);    %Compressor speed, [1/s]     

%%%%%%% STAGE 1 %%%%%%%%%%%%%%%%
s_int1      = s_in;             %Isentropic compression

%Solving for outlet temperature instead of outlet enthalpy. Without this
%fix, it is not possible to compress to a pressure higher than 22 MPa when
%inlet pressure is 1.2 MPa.
options     = optimset('Display', 'off', 'TolX', 1e-4);
F_int       = @(x) Props('S', 'T', x, 'P', p_int*1e3, 'H2') - s_int1;
T_guess     = T_out_last;
T_int1_is   = fsolve(F_int, T_guess, options);
h_int1_is   = Props('H', 'T', T_int1_is, 'P', p_int*1e3, 'H2'); 

%Calculate real outlet enthalpy by using isentropic efficiency, [kJ/kg]
h_int1      = h_in + (h_int1_is - h_in)/eta_is(p_in, p_int);

%Real intermediate temperature, [K]
T_int1      = Props('T', 'H', h_int1, 'P', p_int*1e3, 'H2');

%%%%%%%% INTERCOOLING %%%%%%%%%%%%%
% Cooling down to ambient temperature
h_int2      = Props('H', 'T', T_amb, 'P', p_int*1e3, 'H2');
Q_int       = mflow*(h_int1 - h_int2);

%%%%%%%% STAGE 2 %%%%%%%%%%%
s_int2      = Props('S', 'T', T_amb, 'P', p_int*1e3, 'H2');
s_out       = s_int2;

F_out       = @(x) Props('S', 'T', x, 'P', p_out*1e3, 'H2') - s_out;
T_out_is    = fsolve(F_out, T_guess, options);
h_out_is    = Props('H', 'T', T_out_is, 'P', p_out*1e3, 'H2'); 

%Using isentropic efficiency to calculate real enthalpy
h_out       = h_in + (h_out_is - h_in)/eta_is(p_int, p_out);

%Real outlet temperature, [K]
T_out       = Props('T', 'H', h_out, 'P', p_out*1e3, 'H2');

%%%%%%%% COMPRESSOR WORK %%%%%%%%%%%%%%
W1          = mflow*(h_int1 - h_in);
W2          = mflow*(h_out - h_int1);
W           = W1 + W2;                      %Total compressor work, [kW]

end

