function [ W, mflow, T_out ] = compressor2( p_in, p_out, T_in, T_out_last, N )
% COMPRESSOR2   Calculates the shaft work for compressor 2. The
%               compression is considered adiabatic, and the work 
%               is calculated from the energy balance:        
%                       W   =   m*( h_out(T,p) - h_in(T,p) )
%               The outlet enthalpy, h_out is calculated from the 
%               isentropic efficiency and the outlet isentropic enthalpy:
%                       h_out   = ( h_out,is - h_in )/eta_is + h_in
%               The isentropic enthalpy is then calculated from the 
%               outlet entropy and pressure.
%
% INPUT
% p_in          Inlet pressure, [MPa]
% p_out         Outlet pressure, [MPa]
% T_in          Inlet temperature, [K]
% T_out_last    Outlet temperature, last time step, [K]
% N             Strokes per minute, [1/s]
% 
% OUTPUT
% W             Compressor work, [kW]
% mflow         Mass flow, [kg/s]
% T_out         Outlet temperature, [K]
%
% PARAMETERS
% eta_is        Isentropic efficiency
% eta_v         Volumetric efficiency

%Parameters
Vcyl        = 3.1e-4;               %Cylinder volume, [m3].
c           = 10;                   %Cylinder clearance, [%]

%Heat capacities
Cp          = Props('C', 'P', p_in*1e3, 'T', T_in, 'H2');
Cv          = Props('O', 'P', p_in*1e3, 'T', T_in, 'H2');
gamma       = Cp/Cv;

%Compressibility factors
Zin         = Props('Z', 'T', T_in, 'P', p_in*1e3, 'H2');
Zout        = Props('Z', 'T', T_out_last, 'P', p_out*1e3, 'H2');

%%%%%%%%%% EFFICIENCIES AND TRANSPORT %%%%%%%%%%%%%%%%%

eta_v       = @(p1, p2)     (100 - p2/p1 - c*((p2/p1)^(1/gamma)...
                            *Zin^2/Zout^2 - 1))*1e-2;
eta_is      = @(p1, p2)     0.1091*(log(p2/p1))^3 - ...
                            0.5247*(log(p2/p1))^2 + 0.8577*log(p2/p1)+ ...
                            0.3727;
m_flow      = @(eta, rho)   Vcyl*N*rho*eta;

%Isentropic and volumetric efficiency when out of bound
eta_is_0    = eta_is(1, 1.1);       %Efficiency when p2/p1 = 1.1

%%%%%%%%%%% INLET AND OUTLET CONDITIONS %%%%%%%%%%%%%%%

% Calculate inlet enthalpy and entropy from known T and p.
h_in        = Props('H', 'T', T_in, 'P', p_in*1e3, 'H2');
s_in        = Props('S', 'T', T_in, 'P', p_in*1e3, 'H2');
rho_in      = Props('D', 'T', T_in, 'P', p_in*1e3, 'H2'); 

% Outlet conditions. Isentropic compression.
s_out       = s_in;
h_out_is    = Props('H', 'S', s_out, 'P', p_out*1e3, 'H2'); 


%Calculate real outlet enthalpy by using isentropic efficiency
if p_out/p_in >= 1.1 && p_out/p_in <= 5
    h_out   = h_in + (h_out_is - h_in)/eta_is(p_in, p_out);
    %Mass flow
    mflow       = m_flow(rho_in, eta_v(p_in, p_out));
else
    h_out   = h_in + (h_out_is - h_in)/eta_is_0;
    mflow       = m_flow(rho_in, eta_v(p_in, p_out));
end

%Real temperature out of compressor
T_out       =   Props('T', 'H', h_out, 'P', p_out*1e3, 'H2');

%Calculate compressor work
W           = mflow*(h_out - h_in);

end