function [ P_el, Q_gen ] = electrolysis( mflow, T )
% ELECTROLYSIS  Calculates the power input and heat generated based on
%               the given mass flow.
%               It is assumed that the pressure and temperature in the 
%               stack remain constant, also during off-time.
%               It is assumed that oxygen and hydrogen are ideal gases 
%               under these conditions and that the heat capacities are
%               constant.
%
% INPUT
% mflow       Mass flow demand, [kg/s]
% T           Operating temperature, [K]
%   
% OUTPUT
% P_el    Electical power input, [kW]
% Q_gen   Heat generated, [W]

%Reference temperature [K] and pressure [MPa] (standard conditions)
T_ref       = 273.15 + 25;
p_ref       = 0.1;

%Molar masses, [kg/mol]
Mm_h2       = Props('H2','molemass')*1e-3;
Mm_o2       = Props('O2','molemass')*1e-3;
Mm_h2o      = Props('H2O','molemass')*1e-3;

%Molar heat capacities, [kJ/K mol]
Cp_h2       = Props('C', 'P', p_ref*1e3, 'T', T_ref, 'H2')*Mm_h2;
Cp_o2       = Props('C', 'P', p_ref*1e3, 'T', T_ref, 'O2')*Mm_o2;
Cp_h2o      = Props('C', 'P', p_ref*1e3, 'T', T_ref, 'H2O')*Mm_h2o;

%Formation enthalpy at standard conditions, [kJ/mol]. REF SI Chemical data
H0_h2       = 0;
H0_o2       = 0;
H0_h2o      = -286;             

%Overall electrolyzer efficiency
eta         = 0.5;                      

%Enthalpies at new temperature, [kJ/mol]
H_h2        = H0_h2 + Cp_h2*(T - T_ref);
H_o2        = H0_o2 + Cp_o2*(T - T_ref);
H_h2o       = H0_h2o + Cp_h2o*(T - T_ref);

%Enthalpy of reaction, [kJ/mol]
del_H_rx    = 0.5*H_o2 + H_h2 - H_h2o;

%Power input needed, [kW]
P_el        = mflow*del_H_rx/(Mm_h2*eta);

%Heat generated, [kW]
Q_gen       = (1-eta)*P_el;


end

