function [ y, y0 ] = euler_integrator( mflow4, h_4, ...
                                        h_st, mflow7, h_7, y0 )
% EULER_INTEGRATOR  Explicit euler is used as integrator
%                       y(k + 1)    = y(k)    + del_t*y(k)
%                   where y(k)  = F*v.
%                   y is a vector containing the conserved
%                   quanitites, y = [U , m]', F is the incidence matrix
%                   providing the directionality of the mass flows and v is
%                   a transport vector, v = [m4 , m7]'.
%                   Because of conservation, the outlet flow of the storage
%                   tank is equal to the inlet flow of the vehicle tank. 
%                   
% INPUT
% mflow4            Inlet flow of the storage tank, [kg/s]
% h_4               Inlet specific enthalpy of the storage tank, [kJ/kg]
% h_st              Specific enthalpy in the storage tank, [kJ/kg]
% mflow7            Inlet flow of vehicle tank, [kg/s]
% h_7               Inlet specific enthalpy of vehicle tank, [kJ/kg]
% y0                Values from last time step, y(k).
%
% OUTPUT
% y                 Updated primary states
% y0                New "initial" values for next round.

global del_t

%Incidence matrix
F           = [ 1           -1      ; ...
                h_4         -h_st   ; ...
                0           1       ; ...
                0           h_7     ];

%Transport matrix
v           = [ mflow4  mflow7]';

%Euler integration
f_vector    = F*v;
y           = y0    + del_t*f_vector;
y0          = y;

end

