function [ N2 ] = init_n2( m_flow, p_in, T_in, rho_in )
% INIT_N2       Calcuates an initial N2 that gives the same mass flow
%               as flowing through the reduction valve.
%               At this point, the pressure in the vehicle tank is lower
%               than the pressure in the storage tank. Therefore, the
%               compression ratio (r) is set to 1.
%
% INPUT
% m_flow        Mass flow in reduction valve at switch, [kg/s]
% p_in          Inlet pressure, [MPa]
% T_in          Inlet temperature, [K]
% rho_in        Inlet density, [kg/m3]
%
% OUPUT         
% N2            Compressor speed, [1/s]

global T_amb

%Parameters
Vcyl        = 3.1e-4;   %Cylinder volume, [m3]
c           = 10;       %Clearance, [%]
T_out_last  = T_amb;

%Heat capacities
Cp          = Props('C', 'P', p_in*1e3, 'D', rho_in, 'H2');
Cv          = Props('O', 'P', p_in*1e3, 'D', rho_in, 'H2');
gamma       = Cp/Cv;

%Compression ratio
r           = 1;        %Set outlet pressure equal to inlet pressure 

%Compressibility factors
p_out       = p_in*r;
Zin         = Props('Z', 'T', T_in, 'P', p_in*1e3, 'H2');
Zout        = Props('Z', 'T', T_out_last, 'P', p_out*1e3, 'H2');

%Volumetric efficiency
eta_v       = @(r)     (100 - r - c*((r)^(1/gamma)...
                            *Zin^2/Zout^2 - 1))*1e-2;
                       
%Initial compressor speed
N2          = m_flow/(Vcyl*rho_in*eta_v(r));

end

