% DESCRIPTION
% Dynamic model of a hydrogen refueling station with direct compression of 
% hydrogen gas into the vehicle tank.
% The station consists of an electrolyzer that produces hydrogen from
% water, three heat exchangers whose job is to remove heat that is added in
% compression and expansion processes, two compressors, a storage tank and
% a reduction valve.
% In this version, a heat exchanger has been included after compressor 2.
% The vehicle that connects to the station consists of one hydrogen storage
% tank.
% The mass flows in the station are induced by compressor 1 (mflow2) and 
% the reduction valve/compressor 2 (mflow5A/B). 
% The rest of the mass flows are given by steady state mass balances:
%           mflow1 = mflow2 and mflow4 = mflow3 = mflow2
%           mflow7 = mflow6A/B = mfow5A/B
%
% COMPONENTS in station
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Flows:  1        2       3        4      5A/B       6A/B      7        %   
%     EL ---> HX1 ---> C1 ---> HX2 ---> ST ---> RV/C2 ---> HX3 ---> VT   %
%                                                                        %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% In addition, a heat exchanger (HX4) after C2 
%
%EL: Electrolyzer
% HX: Heat exchanger
% C:  Compressor
% ST: Storage tank
% RV: Reduction valve
% VT: Vehicle tank
%
% ASSUMPTIONS
% No pressure drop in pipelines or in heat exchangers.
% Steady-state operation of heat exchangers.
% Adiabatic compression and expansion in compressors and reduction valve.
% Adiabatic filling of vehicle tank. No heat exchange with environment from
% storage tank.
%
% INTEGRATOR
% Explicit Euler method.
%
% PARAMETERS/VARIABLES
% aprr       Average pressure ramp rate, [MPa/s]
% Cv         Valve coefficient
% h          Specific enthalpy, [kJ/kg]
% i          Loop counter
% Kc         Controller gain
% m          Mass in tank, [kg]
% mflow      Mass flow, [kg/s]
% N          Compressor speed, [1/s]
% rho        Density, [kg/m3]
% t          Time, [s]
% tau_C      Desired closed-loop time constant, [s]
% tau_D      Derivative time constant, [s]
% tau_I      Integral time constant, [s]
% T          Temperature, [K]
% u          Specific internal energy, [kJ/kg]
% U          Internal energy, [kJ]
% V          Tank volume, [m3]
% W          Work flow, [kW]
% Q          Heat flow, [kW]


clear all
close all

global T_amb V_vt del_t aprr


%%%%%%%%%%%%%%%%%%%% INPUT VARIABLES %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The ambient temperature, vehicle tank pressure and volume will be
% different for different vehicles. APRR is decided based on these
% variables. The pressure in storage tank can be optimized to save
% compressor work.
T_amb           = 273 + 15;          %Ambient temperature, [K]
p_vehicle_tank  = 2;                 %Initial pressure in vehicle tank, [MPa]
V_vehicle_tank  = 0.172;             %Volume of vehicle tank, [m3]
p_storage_tank  = 30;                %Initial pressure in storage tank, [MPa]         


%%%%%%%%%%%%%%%%%%%% HYDROGEN VEHICLE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The primary (u and m) and secondary (p, T, rho, h) states of the vehicle
% tank before refueling.
p_vt            = p_vehicle_tank;           
T_vt            = T_amb;                        
V_vt            = V_vehicle_tank;               
rho_vt          = Props('D', 'P', p_vt*1e3, 'T', T_amb, 'H2');
m_vt            = rho_vt*V_vt;
u_vt            = Props('U', 'P', p_vt*1e3, 'T', T_amb, 'H2');
h_vt            = Props('H', 'P', p_vt*1e3, 'T', T_amb, 'H2');

% Filling targets: Target density [kg/m3] and target pressure [MPa] based on
% target density and ambient temperature.
% Switching limit [MPa] from the red. valve to the compressor.
rho_target      = 40.2;   
dp_switch       = 0.5; 

if T_amb == 273 + 10
    p_target    = 72;
    aprr        = 28.2/60;  %Tabulated average pressure ramp rate, [MPa/s]. REF: SAE TIR J2601
elseif T_amb == 273 + 15
    p_target    = 72.6;       
    aprr        = 28.2/60;
elseif T_amb == 273 + 20
    p_target    = 73.2;        
    aprr        = 28.2/60; 
elseif T_amb == 273 + 25
    p_target    = 73.6;        
    aprr        = 28.2/60;  
elseif T_amb == 273 + 30
    p_target    = 74.1;         
    aprr        = 27.4/60;       
else
    disp('Not valid ambient temperature input. The valid options are 10, 15, 20, 25 and 30 Celsius degrees')
    return
end

if p_vt > p_storage_tank - dp_switch 
    p_target        =   69.1;          % When initial pressure is 30 MPa
end
    
disp(['Target pressure in vehicle tank is ', num2str(p_target), ' MPa'])

%%%%%%%%%%%%%%%%%%%% REFUELING STATION %%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Electrolyzer
p_electrolyzer  = 1.2;                    %Operating pressure, [MPa]
T_electrolyzer  = 273 + 90;               %Operating temperature, [K]

%Heat exchangers. Outlet temperatures are specified.
Thx1_out        = T_amb;
Thx2_out        = T_amb;
Thx3_out        = 273 - 40;
Thx4_out        = T_amb;

%Storage tank: Primary (u and m) and secondary states (p, T, rho, h).
p_st            = p_storage_tank;                         
T_st            = T_amb;
V_st            = 3;
rho_st          = Props('D', 'P', p_st*1e3, 'T', T_st, 'H2');
m_st            = rho_st*V_st;   
u_st            = Props('U', 'P', p_st*1e3, 'T', T_st, 'H2');
h_st            = Props('H', 'P', p_st*1e3, 'T', T_st, 'H2');


%Initiation of flows: Secondary states.
T_1             = T_electrolyzer;
p_1             = p_electrolyzer;
T_2             = Thx1_out;
p_2             = p_1;
T_3             = T_amb;
p_3             = p_st;
T_4             = Thx2_out;
p_4             = p_st;
T_5             = T_st;
p_5             = p_st;
T_6             = T_amb;
p_6             = p_vt;
T_7             = Thx3_out;
p_7             = p_vt;

%Initial conditions for integrator
x0              = [ m_st , m_st*u_st , ...
                    m_vt   , m_vt*u_vt ]';
                
%%%%%%%%%%%%%%%%%%%% TIME DEFINITIONS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
t               = 0;                            %Start at t = 0, [s]
del_t           = 0.1;                          %Time steps, [s]
i               = 1;                            %Counter
                
%%%%%%%%%%%%%%%%%%%% CONTROLLERS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Reduction valve controller settings
tauC_rv         = 0.5;

%Integrating
k_rv            = 1.81e5;
Kc_rv           = 1/(k_rv*tauC_rv);
tauI_rv         = 4*tauC_rv;

%PID-controller for reduction valve.
controller1     = @(cv, error, int_error)  ...
                    cv + Kc_rv*(error + 1/tauI_rv*int_error*del_t); 

%Compressor controller settings
tauC_c2         = 2;
k_c2            = 0.077;
Kc_c2           = 1/(k_c2*tauC_c2);
tauI_c2         = 4*tauC_c2;

%PID-controller for compressor 2.
controller2     = @(N, error, int_error)  ...
                    N + Kc_c2*(error + 1/tauI_c2*int_error*del_t);

%Initiation of controllers
p_SP            = p_vt + aprr*del_t;        %Set point for pressure in vehicle tank at next time step
error           = p_SP - p_vt;              %Error between set point and measured variable
int_error       = 0;                        %Integrator error        
flag            = 0;                        %Decides initiation of N2    

Cv0             = 2e-6;     %Bias value for valve coefficient                        
N2              = 0;        %Compressor 2 is off

%%%%%%%%%%%%%%%%%%%% ALLOCATION OF PROFILES %%%%%%%%%%%%%%%%%%%%%%%%%%
t_aprr          = round(p_target/aprr)-20;        %Approximate filling time
len             = t_aprr/del_t;  
tspan           = zeros(len, 1);
aprr_prof       = zeros(len, 1);

pst_prof        = zeros(len, 1);
mst_prof        = zeros(len, 1);
ust_prof        = zeros(len, 1);
hst_prof        = zeros(len, 1);
Tst_prof        = zeros(len, 1);

pvt_prof        = zeros(len, 1);
mvt_prof        = zeros(len, 1);
uvt_prof        = zeros(len, 1);
hvt_prof        = zeros(len, 1);
Tvt_prof        = zeros(len, 1);

mf_2_prof       = zeros(len, 1);
mf_5_prof       = zeros(len, 1);

Pel_prof        = zeros(len, 1);
Qel_prof        = zeros(len, 1);
T3_prof         = zeros(len, 1);
Tc1_int_prof    = zeros(len, 1);
Qc1_prof        = zeros(len, 1);
Wc1_prof        = zeros(len, 1);
T6_prof         = zeros(len, 1);
Wc2_prof        = zeros(len, 1);
Qhx1_prof       = zeros(len, 1);
Qhx2_prof       = zeros(len, 1);
Qhx3_prof       = zeros(len, 1);
Qhx4_prof       = zeros(len, 1);

Cv_prof         = zeros(len, 1);
N2_prof         = zeros(len, 1);

%Initial profiles at t = 0 (i = 1)
tspan(1)        = t;
aprr_prof(1)    = p_vt;

pst_prof(1)     = p_st;
mst_prof(1)     = m_st;
ust_prof(1)     = u_st;
hst_prof(1)     = h_st;
Tst_prof(1)     = T_st;

pvt_prof(1)     = p_vt;
mvt_prof(1)     = m_vt;
uvt_prof(1)     = u_vt;
hvt_prof(1)     = h_vt;
Tvt_prof(1)     = T_vt;

T3_prof(1)      = T_3;
Tc1_int_prof(1) = T_2;
T6_prof(1)      = T_6;


while p_vt < p_target 
    
    %Mass flow and work in compressor 1. Sets mass flow in production line.
    [ W_compressor1, mflow2, T_3, Tc1_int, Qc1_int ]    ...
                    = compressor1( p_2, p_3, T_2, T_3 );                       
    mflow3          = mflow2;
    
    %Electrolyzer
    mflow1          = mflow2;
    [ P_electrolyzer, Q_electrolyzer ] ...
                    = electrolysis( mflow1, T_electrolyzer );
    
    %Heat exchanger 1
    [ Q_hx1, h_2 ] ...
                    = heatexchanger( mflow1, T_1, p_1, Thx1_out );
 
    %Heat exchanger 2
    [ Q_hx2, h_4 ] ...
                    = heatexchanger( mflow3, T_3, p_3, Thx2_out );               
    mflow4          = mflow3;
    
    
    %Switching between reduction valve and compressor
    if p_st - p_vt >= dp_switch     
        
        %Calculating new valve coefficient from controller
        Cv          = controller1( Cv0, error, int_error );         
        
        %Mass flow in reduction valve
        [ mflow5, T_6 ]  ...
                    = red_valve( p_5, T_5, rho_st, p_6, Cv );                    
        mflow6      = mflow5;
        
        W_compressor2 = 0;
        Q_hx4         = 0;
        
    elseif p_st - p_vt < 0 
        
        %Controller
        N2          = controller2( N20, error, int_error );      
        
        %Work, mass flow and outlet temp. in compressor 2
        [ W_compressor2, mflow5, T_compressor2 ]    ...
                    = compressor2( p_5, p_6, T_5, T_6, N2 );                          
        %T_6         = T_compressor2;             
        mflow6      = mflow5;
        
        %Heat exchanger after compressor (heat exchanger 4)
        [ Q_hx4, h_6 ] ...
                    = heatexchanger( mflow6, T_compressor2, p_6, Thx4_out );  
        T_6         = Thx4_out;
        
    else
    %At the switch, the pressure in the vehicle tank is lower than the
    %pressure in the storage tank. Therefore, the outlet pressure of the
    %compressor is set equal to the inlet pressure
        
        if flag == 0
            %Initiation of N2 the first time the compressor runs. N2 is 
            %initiated based on the mass flow in the reduction valve when
            %the switch occurs.
            
            Cv      = 0;                         %Closing valve
            N2      = init_n2(mflow6, p_st, T_st, rho_st); 
            N20     = N2;
            flag    = 1;                        %Turns off N2 initiation 
            
            time    = tspan(i);
            disp(['Change from reduction valve to compressor at ', ...
                                                    num2str(time), ' s'])
            disp(['Initial N2 is ', num2str(N2), ' strokes per second'])
            
        else
            
            %Controller calculating compressor speed           
            N2      = controller2( N20, error, int_error ); 
            
        end
        
        %Work, mass flow and outlet temp. in compressor 2
        %Assume outlet pressure is equal to inlet pressure  
        [ W_compressor2, mflow5, T_compressor2 ]    ...
                    = compressor2( p_5, p_5, T_5, T_6, N2 );                                            
        %T_6   = T_compressor2;             
        mflow6      = mflow5; 
        
        %Heat exchanger after compressor (heat exchanger 4)
        %Assume inlet temperature is equal to ambient temperature to avoid
        %negative heat rate 
        Thx4_in     = T_amb;
        [ Q_hx4, h_6 ] ...
                    = heatexchanger( mflow6, Thx4_in, p_6, Thx4_out );  
        T_6         = Thx4_out;
    end
    
    %Heat exchanger 3
    [ Q_hx3, h_7 ]  = heatexchanger( mflow6, T_6, p_6, Thx3_out );                           
    mflow7          = mflow6;
    
    %Integration - Explicit Euler
    [ x, x0 ]       = euler_integrator( mflow4, h_4, ...             
                                        h_st, mflow7, h_7, x0 );
    
    %Calculating new primary and secondary states in tanks
    m_st            = x(1);
    u_st            = x(2)/m_st;
    rho_st          = m_st/V_st;
    T_st            = T_solver(u_st, rho_st, T_st);
    p_st            = Props('P', 'D', rho_st, 'T', T_st, 'H2')/(1e3);       
    h_st            = Props('H', 'P', p_st*1e3, 'T', T_st, 'H2');
    
    m_vt            = x(3);
    u_vt            = x(4)/m_vt;
    rho_vt          = m_vt/V_vt;
    T_vt            = T_solver(u_vt, rho_vt, T_vt);                             
    p_vt            = Props('P', 'D', rho_vt, 'T', T_vt, 'H2')/(1e3);
    h_vt            = Props('H', 'P', p_vt*1e3, 'T', T_vt, 'H2');
        
    %Updating flows 
    p_4             = p_st;
    p_3             = p_st;
    T_5             = T_st;
    p_5             = p_st;
    p_6             = p_vt;
    p_7             = p_vt;
   
    %Updating profiles
    tspan(i + 1)    = t + del_t;        %tspan(i) + del_t;
    t               = tspan(i + 1);
    aprr_prof(i + 1)= p_SP;
      
    pst_prof(i + 1) = p_st;
    mst_prof(i + 1) = m_st;
    ust_prof(i + 1) = u_st;
    hst_prof(i + 1) = h_st;
    Tst_prof(i + 1) = T_st;
    
    pvt_prof(i + 1) = p_vt;
    mvt_prof(i + 1) = m_vt;
    uvt_prof(i + 1) = u_vt;
    hvt_prof(i + 1) = h_vt;
    Tvt_prof(i + 1) = T_vt;
    
    mf_2_prof(i + 1) = mflow2; 
    mf_5_prof(i + 1) = mflow7;
    
    Pel_prof(i + 1) = P_electrolyzer;
    Qel_prof(i + 1) = Q_electrolyzer;
    T3_prof(i + 1)  = T_3;
    Tc1_int_prof(i + 1) = Tc1_int;
    T6_prof(i + 1) = T_6;
    Qhx1_prof(i + 1)= Q_hx1;
    Qc1_prof(i + 1) = Qc1_int;
    Qhx2_prof(i + 1)= Q_hx2;
    Qhx3_prof(i + 1)= Q_hx3;
    Qhx4_prof(i + 1)= Q_hx4;
    Wc1_prof(i + 1) = W_compressor1;
    Wc2_prof(i + 1) = W_compressor2;
    
    Cv_prof(i + 1)  = Cv;
    N2_prof(i + 1)  = N2;
       
    %Updating error for controller action
    p_SP            = p_SP + aprr*del_t;    %Oppdatere fr eller etter oppdatering av kontroller????                
    error           = p_SP - p_vt;
    int_error       = int_error + error;
    
    i               = i + 1;   
    
end

%%%%%%%%%%%%%%%%%%%% STATE OF CHARGE %%%%%%%%%%%%%%%%%%%%%%%%%
soc                 = rho_vt/rho_target*100;

disp(['Final pressure in vehicle tank ', num2str(p_vt), ' MPa'])
disp(['Final mass in vehicle tank ', num2str(m_vt), ' kg'])
disp(['The state of charge is ', num2str(soc), '%'])

%%%%%%%%%%%%%%%%%%%% MAXIMUM AND AVERAGES %%%%%%%%%%%%%%%%%%%%
[Qhx3_max, k]       = max(Qhx3_prof);

Qhx2_at_max         = Qhx2_prof(k);
Thx2_at_max         = T3_prof(k);
Qc1_at_max          = Qc1_prof(k);
Tc1_at_max          = Tc1_int_prof(k);
Qel_at_max          = Qel_prof(k);

Pel_avg             = sum(Pel_prof)/(length(Pel_prof));
Qel_avg             = sum(Qel_prof)/length(Qel_prof);
Qhx1_avg            = sum(Qhx1_prof)/length(Qhx1_prof);
Qc1_avg             = sum(Qc1_prof)/length(Qc1_prof);
Tc1_avg             = sum(Tc1_int_prof)/length(Tc1_int_prof);
Qhx2_avg            = sum(Qhx2_prof)/length(Qhx2_prof);
Thx2_avg            = sum(T3_prof)/length(T3_prof);
Qhx3_avg            = sum(Qhx3_prof)/length(Qhx3_prof);
Qhx4_avg            = sum(Qhx4_prof)/length(Qhx4_prof);
Wc1_avg             = sum(Wc1_prof)/length(Wc1_prof);
Wc2_avg             = sum(Wc2_prof)/length(Wc2_prof);

Q_tot               = Qel_avg + Qhx1_avg + Qhx2_avg + Qc1_avg + Qhx3_avg;
W_tot               = Pel_avg + Wc1_avg + Wc2_avg;

avg                 = [Pel_avg Wc1_avg Wc2_avg ...
                        Qel_avg Qhx1_avg Qc1_avg ...
                            Qhx2_avg Qhx3_avg Qhx4_avg]';

%%%%%%%%%%%%%%%%%%%% PLOTS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
figure(1)
plot(tspan, pst_prof, tspan, pvt_prof, tspan, aprr_prof)
xlabel('Time, [s]')
ylabel('Pressure, [MPa]')
legend('Storage tank', 'Vehicle tank', 'APRR')

figure(2)
plot(tspan, mst_prof, tspan, mvt_prof)
xlabel('Time [s]')
ylabel('Mass[kg]')
legend('Storage tank', 'Vehicle tank')

figure(3)
plot(tspan, mf_2_prof, tspan, mf_5_prof)
xlabel('Time [s]')
ylabel('Mass flow [kg/s]')
legend('Mass flow compressor 1', 'Mass flow compressor2/reduction valve')

figure(4)
plot(tspan, ust_prof, tspan, uvt_prof)
xlabel('Time [s]')
ylabel('Specific internal energy, u [kJ/kg]')
legend('Storage tank', 'Vehicle tank')

figure(5)
plot(tspan, hst_prof, tspan, hvt_prof)
xlabel('Time [s]')
ylabel('Specific enthalpy, h [kJ/kg]')
legend('Storage tank', 'Vehicle tank')

figure(6)
plot(tspan, Tvt_prof, tspan, Tst_prof, tspan, T3_prof, tspan, T6_prof, tspan, Tc1_int_prof)
xlabel('Time [s]')
ylabel('Temperature[K]')
legend('Vehicle tank', 'Storage tank', 'Compressor 1', 'Reduction valve/compressor 2', 'Intercooling compressor 1')

figure(7)
plot(tspan, Qhx1_prof, tspan, Qhx2_prof, tspan, Qhx3_prof, tspan, Qc1_prof, tspan, Qhx4_prof)
xlabel('Time [s]')
ylabel('Heat removed [kW]')
legend('Heat removal HX1', 'Heat removal HX2', 'Heat removal HX3', 'Heat removal intercooler', 'Heat removal HX4')

figure(8)
plot(tspan, Wc1_prof, tspan, Wc2_prof)
xlabel('Time [s]')
ylabel('Compressor work [kW]')
legend('Compressor 1', 'Compressor 2')







